Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports TcpMessages

Namespace AdvCallManager
	''' <summary>
	''' CallDispatcheer manage a list of RepConnection objects one per each registered
	''' call center rep. 
	''' </summary>
	Friend Class CallDispatcher
		Private _repList As Dictionary(Of String, CallRep) = New Dictionary(Of String, CallRep)()
		Private _callQueue As New CallQueue()
		Public Sub New()
			'register call queue listener
			AddHandler _callQueue.OnCallReceived, AddressOf DispatchCall

		End Sub
		Private Sub DispatchCall(ByVal sender As Object, ByVal args As EventArgs)
			Dim customerCall As PhoneCall = _callQueue.Dequeue()
			Dim rep As CallRep = GetAvailableRep()
			If rep Is Nothing Then
				Console.WriteLine("Rep is not available...")
				Return
			End If
			rep.Locked = True
			rep.LockTime = DateTime.Now
			Dim callMsg As New CallMessage(MessageAction.Call)
			callMsg.CallerName = customerCall.Name
			callMsg.CallerPhone = customerCall.PhoneNumber
			callMsg.IvrData = customerCall.OrderNumber
			rep.RepConnection.SendAsync(callMsg)
			Console.WriteLine("forwarding call to the rep...")
		End Sub
		Public Function GetAvailableRep() As CallRep
			For Each key As String In _repList.Keys
				Dim rep As CallRep = _repList(key)
				If (Not rep.Available) Then
					Continue For
				End If
				If rep.Locked Then
					Continue For
				End If
				'don't dispatch if a rep just rejected the call
				If Not(rep.LastRejectTime = DateTime.MaxValue) Then
					'rep recently rejected the call. let us see if 2 minutes elapsed since 
					If rep.LastRejectTime.AddMinutes(2.0) > DateTime.Now Then
						'time has not elapsed
						Continue For
					End If
				End If
				Return rep
			Next key
			Return Nothing
		End Function


		Public ReadOnly Property RepList() As Dictionary(Of String, CallRep)
			Get
				Return _repList
			End Get
		End Property

		Public ReadOnly Property CustomerCallQueue() As CallQueue
			Get
				Return _callQueue
			End Get
		End Property
		Public Sub CleanUp()
			For Each key As String In _repList.Keys
				Dim rep As CallRep = _repList(key)
				rep.RepConnection.AcceptedSocket.Shutdown(SocketShutdown.Both)
				rep.RepConnection.AcceptedSocket.Close()
			Next key
		End Sub
	End Class
End Namespace
