Imports Microsoft.VisualBasic
Imports System
Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports TcpMessages

Namespace AdvCallManager
	Friend Class CallCenterServer
		'public static CallDispatcher RepDispatcher = new CallDispatcher();
		Private _repDispatcher As CallDispatcher
		Private _listener As Socket
		Private _socketAcceptArgs As New SocketAsyncEventArgs()
		Public Sub New()
			_repDispatcher = New CallDispatcher()
			AddHandler _socketAcceptArgs.Completed, AddressOf AcceptAsyncCallback

		End Sub
		Public Sub Start(ByVal port As Integer)
			Dim ipHostInfo As IPHostEntry = Dns.GetHostEntry(Dns.GetHostName())
			'Should be within the port range of 4502-4532
			Dim localEndPoint As New IPEndPoint(IPAddress.Any, port)

			_listener = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)

			' Bind the socket to the local endpoint and listen for incoming connections.
			Try
				_listener.Bind(localEndPoint)
				_listener.Listen(50)
				AcceptAsync()
			Catch e As Exception
				Console.WriteLine(e.ToString())
			End Try
		End Sub
		Private Sub AcceptAsync()
			AcceptAsync(_socketAcceptArgs)
		End Sub

		Private Sub AcceptAsync(ByVal socketAcceptArgs As SocketAsyncEventArgs)

			If (Not _listener.AcceptAsync(socketAcceptArgs)) Then
				AcceptAsyncCallback(socketAcceptArgs.AcceptSocket, socketAcceptArgs)
			End If
		End Sub

		Private Sub AcceptAsyncCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.SocketError = SocketError.Success Then
				Console.WriteLine("Connection accepted...")
				Dim repCon As New RepConnection(e.AcceptSocket)
				AddHandler repCon.OnReceive, AddressOf OnReceiveCallback
				'schedule the first receive
				repCon.ReceiveAsync()
			End If
			'the following statement is necessary for the reuse of _socketAcceptArgs
			e.AcceptSocket = Nothing
			'schedule a new accept request
			AcceptAsync()
		End Sub

		Private Sub OnReceiveCallback(ByVal sender As Object, ByVal e As ReceiveArgs)
			Dim repCon As RepConnection = CType(sender, RepConnection)
			ProcessMessage(e.Result, repCon)
		End Sub

		Public Sub ProcessMessage(ByVal msg As NetworkMessage, ByVal repCon As RepConnection)
			Select Case msg.Action
				Case MessageAction.Register
						Dim registerMsg As RegisterMessage = CType(msg, RegisterMessage)
						Dim rep As New CallRep(repCon)
						Dim ts As DateTime = DateTime.Now
						rep.RepName = registerMsg.RepName
						rep.RepNumber = registerMsg.RepNumber
						rep.PhoneExtension = registerMsg.PhoneExtension
						rep.ShiftStartTime = ts
						rep.Available = True
						'<todo> make thread safety changes
						_repDispatcher.RepList.Add(rep.RepNumber, rep)
						Dim registerResponse As New RegisterResponse(MessageAction.RegisterResponse)
						registerResponse.RepNumber = registerMsg.RepNumber
						registerResponse.RegistrationTimestamp = ts
						registerResponse.CallManagerServerName = Dns.GetHostName()
						Console.WriteLine("sending resonse...")
						rep.RepConnection.SendAsync(registerResponse)
				Case MessageAction.Unregister
						Dim unregisterMsg As RepKeyMessage = CType(msg, RepKeyMessage)
						Dim repKey As String = unregisterMsg.RepNumber
						Dim rep As CallRep = _repDispatcher.RepList(repKey)
						If rep IsNot Nothing Then
							SyncLock _repDispatcher.RepList
								_repDispatcher.RepList.Remove(repKey)
								Console.WriteLine("Unregistering {0}, {1}, {2}", rep.RepName, rep.RepNumber, rep.PhoneExtension)
							End SyncLock
						End If
				Case MessageAction.Accept
					Dim acceptMsg As RepKeyMessage = TryCast(msg, RepKeyMessage)
					Dim r1 As CallRep = _repDispatcher.RepList(acceptMsg.RepNumber)
					r1.Available = False
					r1.Locked = False
					r1.LockTime = DateTime.MaxValue
					r1.LastRejectTime = DateTime.MaxValue

				Case MessageAction.Reject
					Dim rejectMsg As RepKeyMessage = TryCast(msg, RepKeyMessage)
					Dim r2 As CallRep = _repDispatcher.RepList(rejectMsg.RepNumber)
					'keep it locked for some time to avoid resending the request to the same rep
					r2.Available = True
					r2.Locked = False
					r2.LockTime = DateTime.MaxValue
					r2.LastRejectTime = DateTime.Now
				'this will only true for the simulator
				Case MessageAction.Call
					'receive the call
					Dim callMsg As CallMessage = TryCast(msg, CallMessage)
					Console.WriteLine(String.Format("Received {0}'s call on {1}", callMsg.CallerName, callMsg.CallerPhone))
					Dim phoneCall As New PhoneCall()
					phoneCall.Name = callMsg.CallerName
					phoneCall.PhoneNumber = callMsg.CallerPhone
					phoneCall.OrderNumber = callMsg.IvrData
					_repDispatcher.CustomerCallQueue.Enqueue(phoneCall)
				Case MessageAction.FinishCall
					Dim finishCallMsg As RepKeyMessage = TryCast(msg, RepKeyMessage)
					Dim finishCallRep As CallRep = _repDispatcher.RepList(finishCallMsg.RepNumber)
					If finishCallRep IsNot Nothing Then
						finishCallRep.Available = True
						finishCallRep.Locked = False
						finishCallRep.LockTime = DateTime.MaxValue
						finishCallRep.LastRejectTime = DateTime.MaxValue
						finishCallRep.NoOfCallsServiced += 1
						Console.WriteLine("Finishing the call {0}, {1}, {2}", finishCallRep.RepName, finishCallRep.RepNumber, finishCallRep.PhoneExtension)
					End If
				Case Else
					Throw New ApplicationException(String.Format("Unknowns message type {0}",msg.Action.ToString()))
			End Select
		End Sub
		Public Sub [Stop]()
			'notify all the connected reps
			_repDispatcher.CleanUp()
			_listener.Close()
		End Sub
	End Class
''' <summary>
''' RepConnection represents the accepted socket and the associated receive buffer. 
''' The CallDispatcher will maintain a list of RepConnection objects with live sockets 
''' for pushing notifications. RepConnection does not implement the client disconnect fucntionality.
''' </summary>

	Friend Class RepConnection
		Private _receiveBuffer As ReceiveBuffer
		Private _acceptedSocket As Socket
		Public Event OnReceive As EventHandler(Of ReceiveArgs)
		Private _receiveSocketArgs As SocketAsyncEventArgs

		Public Property ReceiveBuffer() As ReceiveBuffer
			Get
				Return _receiveBuffer
			End Get
			Set(ByVal value As ReceiveBuffer)
				_receiveBuffer = value
			End Set
		End Property
		Public Sub New(ByVal acceptedSocket As Socket)
			Me._acceptedSocket = acceptedSocket
			Me.ReceiveBuffer = New ReceiveBuffer()
			Me._receiveSocketArgs = New SocketAsyncEventArgs()
			Me._receiveSocketArgs.SetBuffer(_receiveBuffer.Buffer, 0, _receiveBuffer.Buffer.Length)
			AddHandler _receiveSocketArgs.Completed, AddressOf ReceiveCallback
		End Sub
		Public Property AcceptedSocket() As Socket
			Get
				Return _acceptedSocket
			End Get
			Set(ByVal value As Socket)
				_acceptedSocket = value
			End Set
		End Property
		'receives a complete message
		Public Function ReceiveSync() As NetworkMessage
			Do
				Dim bytesReceived As Integer = _acceptedSocket.Receive(ReceiveBuffer.Buffer, ReceiveBuffer.Offset, ReceiveBuffer.Remaining, SocketFlags.None)
				'increment the offset
				_receiveBuffer.Offset += bytesReceived

				'check if a complete message is already received
				If _receiveBuffer.IsMessageReceived() Then
					Exit Do
				End If
			Loop
			'deseriaize the message
			Dim msg As NetworkMessage = NetworkMessage.Deserialize(ReceiveBuffer.Buffer)
			'adjust the buffer
			ReceiveBuffer.AdjustBuffer()
			Return msg
		End Function

		Public Sub ReceiveAsync()
			ReceiveAsync(_receiveSocketArgs)
		End Sub

		Private Sub ReceiveAsync(ByVal socketAsyncEventArgs As SocketAsyncEventArgs)
			If (Not _acceptedSocket.ReceiveAsync(socketAsyncEventArgs)) Then
				ReceiveCallback(_acceptedSocket, socketAsyncEventArgs)
			End If
		End Sub

		Private Sub ReceiveCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.SocketError <> SocketError.Success Then
				Return
			End If
			_receiveBuffer.Offset += e.BytesTransferred
			If _receiveBuffer.IsMessageReceived() Then
				If OnReceiveEvent IsNot Nothing Then
					Dim msg As NetworkMessage = NetworkMessage.Deserialize(_receiveBuffer.Buffer)
					_receiveBuffer.AdjustBuffer()
					RaiseEvent OnReceive(Me, New ReceiveArgs(msg))
				End If
			Else
				'adjust the buffer pointer
				e.SetBuffer(_receiveBuffer.Offset, _receiveBuffer.Remaining)
			End If
			'queue a an async read request
			ReceiveAsync(_receiveSocketArgs)
		End Sub

		Public Sub SendSync(ByVal msg As NetworkMessage)
			Dim numBytes As Integer = _acceptedSocket.Send(NetworkMessage.Serialize(msg))
		End Sub

		Public Sub SendAsync(ByVal msg As NetworkMessage)
			Dim sendSocketArgs As New SocketAsyncEventArgs()
			Dim sendBytes() As Byte = NetworkMessage.Serialize(msg)
			sendSocketArgs.SetBuffer(sendBytes, 0, sendBytes.Length)
			AddHandler sendSocketArgs.Completed, AddressOf SendCallback
			SendAsync(sendSocketArgs)
		End Sub

		Private Sub SendAsync(ByVal sendSocketArgs As SocketAsyncEventArgs)
			If (Not _acceptedSocket.SendAsync(sendSocketArgs)) Then
				'call is completed synchronously
				SendCallback(_acceptedSocket, sendSocketArgs)
			End If
		End Sub

		Private Sub SendCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.BytesTransferred < e.Count Then
				'some more bytes to send
				e.SetBuffer(e.BytesTransferred, e.Count - e.BytesTransferred)
				SendAsync(e)
			End If
		End Sub
	End Class
	Public Class ReceiveArgs
		Inherits EventArgs
		Private _msg As NetworkMessage = Nothing
		Public Sub New(ByVal msg As NetworkMessage)
			Me._msg = msg
		End Sub
		Public ReadOnly Property Result() As NetworkMessage
			Get
				Return _msg
			End Get
		End Property
	End Class

End Namespace
