Imports Microsoft.VisualBasic
Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Windows
Imports TcpMessages

Namespace AdvClientLibrary
	Public Class ReceiveArgs
		Inherits EventArgs
		Private _msg As NetworkMessage = Nothing
		Public Sub New(ByVal msg As NetworkMessage)
			Me._msg = msg
		End Sub
		Public ReadOnly Property Result() As NetworkMessage
			Get
				Return _msg
			End Get
		End Property
	End Class
	Public Class CallNetworkClient
		Private _socket As Socket
		Private _receiveBuffer As ReceiveBuffer

		Public Event OnConnectError As EventHandler(Of EventArgs)
		Public Event OnReceive As EventHandler(Of ReceiveArgs)
		Public _receiveSocketArgs As SocketAsyncEventArgs
		Public _sendSocketArgs As SocketAsyncEventArgs

		Public Sub New()
			_receiveBuffer = New ReceiveBuffer()
			_receiveSocketArgs = New SocketAsyncEventArgs()
			_sendSocketArgs = New SocketAsyncEventArgs()
			_receiveSocketArgs.SetBuffer(_receiveBuffer.Buffer, _receiveBuffer.Offset, _receiveBuffer.Remaining)
			AddHandler _receiveSocketArgs.Completed, AddressOf ReceiveCallback
			AddHandler _sendSocketArgs.Completed, AddressOf SendCallback
		End Sub
		Public ReadOnly Property RecieveSocketArgs() As SocketAsyncEventArgs
			Get
				Return _receiveSocketArgs
			End Get
		End Property
		Public ReadOnly Property SendSocketArgs() As SocketAsyncEventArgs
			Get
				Return _sendSocketArgs
			End Get
		End Property
		Public ReadOnly Property ConnectedSocket() As Socket
			Get
				Return _socket
			End Get
		End Property
		Public Sub Connect(ByVal portNo As Integer)
			_socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
			Dim connectEventArgs As New SocketAsyncEventArgs()
			AddHandler connectEventArgs.Completed, AddressOf connectEventArgs_Completed
			connectEventArgs.RemoteEndPoint = New DnsEndPoint(Application.Current.Host.Source.Host, portNo)
			_socket.ConnectAsync(connectEventArgs)
		End Sub
		Private Sub connectEventArgs_Completed(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.SocketError = SocketError.Success Then
				'drop a receive request in the thread pool 
				ReceiveAsync()
			Else
				RaiseEvent OnConnectError(sender, e)
			End If
		End Sub

		Public Sub ReceiveAsync()
		   ReceiveAsync(_receiveSocketArgs)
		End Sub

		Private Sub ReceiveAsync(ByVal recvArgs As SocketAsyncEventArgs)
			If (Not _socket.ReceiveAsync(recvArgs)) Then
				ReceiveCallback(_socket, recvArgs)
			End If
		End Sub

		Private Sub ReceiveCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.SocketError <> SocketError.Success Then
				Return
			End If

			_receiveBuffer.Offset += e.BytesTransferred
			If _receiveBuffer.IsMessageReceived() Then
				If OnReceiveEvent IsNot Nothing Then
					Dim msg As NetworkMessage = NetworkMessage.Deserialize(_receiveBuffer.Buffer)
					_receiveBuffer.AdjustBuffer()
					RaiseEvent OnReceive(Me, New ReceiveArgs(msg))
				End If
			Else
				'adjust the buffer pointer
				e.SetBuffer(_receiveBuffer.Offset, _receiveBuffer.Remaining)
			End If
			'drop a receive request into the threadpool
			ReceiveAsync()
		End Sub

		Public Sub SendAsync(ByVal msg As NetworkMessage)
			Dim sendBytes() As Byte = NetworkMessage.Serialize(msg)
			_sendSocketArgs.SetBuffer(sendBytes, 0, sendBytes.Length)
			SendAsync(_sendSocketArgs)
		End Sub

		Private Sub SendAsync(ByVal sendSocketArgs As SocketAsyncEventArgs)
			If (Not _socket.SendAsync(sendSocketArgs)) Then
				'call is completed synchronously
				SendCallback(Me._socket, sendSocketArgs)
			End If
		End Sub

		Private Sub SendCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.SocketError <> SocketError.Success Then
				'do error management here
				Return
			End If
			If e.BytesTransferred < e.Count Then
				'send is not complete more bytes to send
				e.SetBuffer(e.BytesTransferred, e.Count - e.BytesTransferred)
				SendAsync(e)
			End If
		End Sub

	End Class
End Namespace