'CAUTION:This group of classes will be used on the Silverlight Client also
'so, please make sure of the compatibility at the source code level

Imports Microsoft.VisualBasic
Imports System
Imports System.Text
Imports System.Xml.Serialization
Imports System.IO

Namespace TcpMessages
	Public Enum MessageAction
		InvalidAction
		Register
		RegisterResponse
		Unregister
		UnRegisterResponse
		[Call]
		FinishCall
		Accept
		Reject
	End Enum
	<XmlInclude(GetType(RegisterMessage)), XmlInclude(GetType(RegisterResponse)), XmlInclude(GetType(CallMessage)), XmlInclude(GetType(RepKeyMessage))> _
	Public MustInherit Class NetworkMessage
		Public Shared ReadOnly LENGTH_BYTES As Integer = 4
		Protected _messageAction As MessageAction
		Public Property Action() As MessageAction
			Get
				Return _messageAction
			End Get
			Set(ByVal value As MessageAction)
				_messageAction = value
			End Set
		End Property
		'first 4 bytes of the buffer will contain the length of the serialized NetworkMessage
		'this function treats input argument as immutable
		Public Shared Function Deserialize(ByVal buffer() As Byte) As NetworkMessage
			Dim length As Integer = BitConverter.ToInt32(buffer, 0)
			Dim startIndex As Integer = Len(New Integer)
			Dim strNetworkMessage As String = Encoding.UTF8.GetString(buffer, startIndex, length)

			Dim stringReader As New StringReader(strNetworkMessage)
			Dim xmlSerializer As New XmlSerializer(GetType(NetworkMessage))
			Return CType(xmlSerializer.Deserialize(stringReader), NetworkMessage)
		End Function

		'the first 4 bytes of the serialized byte array will contain the length of the buffer that
		'contains the previously serialized NetworkMessage;
		Public Shared Function Serialize(ByVal msg As NetworkMessage) As Byte()
			Dim stringWriter As New StringWriter()
			Dim xmlSerializer As New XmlSerializer(GetType(NetworkMessage))
			xmlSerializer.Serialize(stringWriter, msg)
			Dim messageBytes() As Byte = Encoding.UTF8.GetBytes(stringWriter.ToString())
			Dim lengthBytes() As Byte = BitConverter.GetBytes(messageBytes.Length)
			Dim finalBytes(lengthBytes.Length + messageBytes.Length - 1) As Byte
			'copy the length of the serialized object
			Array.Copy(lengthBytes, 0, finalBytes, 0, lengthBytes.Length)
			Array.Copy(messageBytes, 0, finalBytes, lengthBytes.Length, BitConverter.ToInt32(lengthBytes, 0))
			Return finalBytes
		End Function

	End Class

	Public Class RegisterMessage
		Inherits NetworkMessage
		Public Sub New()
			_messageAction = MessageAction.InvalidAction
		End Sub
		Public Sub New(ByVal action As MessageAction)
			_messageAction = action
		End Sub

		Private privateRepName As String
		Public Property RepName() As String
			Get
				Return privateRepName
			End Get
			Set(ByVal value As String)
				privateRepName = value
			End Set
		End Property
		Private privateRepNumber As String
		Public Property RepNumber() As String
			Get
				Return privateRepNumber
			End Get
			Set(ByVal value As String)
				privateRepNumber = value
			End Set
		End Property
		Private privatePhoneExtension As Integer
		Public Property PhoneExtension() As Integer
			Get
				Return privatePhoneExtension
			End Get
			Set(ByVal value As Integer)
				privatePhoneExtension = value
			End Set
		End Property

	End Class
	Public Class RegisterResponse
		Inherits NetworkMessage
		 Public Sub New()
			_messageAction = MessageAction.InvalidAction
		 End Sub
		 Public Sub New(ByVal action As MessageAction)
			_messageAction = action
		 End Sub
		Private privateRepNumber As String
		Public Property RepNumber() As String
			Get
				Return privateRepNumber
			End Get
			Set(ByVal value As String)
				privateRepNumber = value
			End Set
		End Property
		Private privateCallManagerServerName As String
		Public Property CallManagerServerName() As String
			Get
				Return privateCallManagerServerName
			End Get
			Set(ByVal value As String)
				privateCallManagerServerName = value
			End Set
		End Property
		Private privateRegistrationTimestamp As DateTime
		Public Property RegistrationTimestamp() As DateTime
			Get
				Return privateRegistrationTimestamp
			End Get
			Set(ByVal value As DateTime)
				privateRegistrationTimestamp = value
			End Set
		End Property
	End Class

	Public Class CallMessage
		Inherits NetworkMessage
		Public Sub New()
			_messageAction = MessageAction.InvalidAction
		End Sub
		Public Sub New(ByVal action As MessageAction)
			_messageAction = action
		End Sub

		Private privateCallerName As String
		Public Property CallerName() As String
			Get
				Return privateCallerName
			End Get
			Set(ByVal value As String)
				privateCallerName = value
			End Set
		End Property
		Private privateCallerPhone As String
		Public Property CallerPhone() As String
			Get
				Return privateCallerPhone
			End Get
			Set(ByVal value As String)
				privateCallerPhone = value
			End Set
		End Property
		Private privateIvrData As String
		Public Property IvrData() As String
			Get
				Return privateIvrData
			End Get
			Set(ByVal value As String)
				privateIvrData = value
			End Set
		End Property
	End Class

	Public Class RepKeyMessage
		Inherits NetworkMessage
		Public Sub New()
			_messageAction = MessageAction.InvalidAction
		End Sub
		Public Sub New(ByVal action As MessageAction)
			_messageAction = action
		End Sub
		Private privateRepNumber As String
		Public Property RepNumber() As String
			Get
				Return privateRepNumber
			End Get
			Set(ByVal value As String)
				privateRepNumber = value
			End Set
		End Property
	End Class

	Friend Class SocketBuffer
		Public Const BUFFERSIZE As Integer = 32768
		Protected _buffer(BUFFERSIZE - 1) As Byte
		Protected _offset As Integer = 0

		Public Property Buffer() As Byte()
			Get
				Return _buffer
			End Get
			Set(ByVal value As Byte())
				_buffer = value
			End Set
		End Property
		'offset will allways indicate the length of the buffer that is filled
		Public Property Offset() As Integer
			Get
				Return _offset
			End Get
			Set(ByVal value As Integer)
				_offset = value
			End Set
		End Property

		Public ReadOnly Property Remaining() As Integer
			Get
				Return _buffer.Length - _offset
			End Get
		End Property
	End Class

	Friend Class ReceiveBuffer
		Inherits SocketBuffer
		'removes a serlialized message from the buffer, copies the partial message to the begining
		'and adjusts the offset
		Public Sub AdjustBuffer()
			Dim messageSize As Integer = BitConverter.ToInt32(_buffer, 0)
			Dim lengthToCopy As Integer = _offset - NetworkMessage.LENGTH_BYTES - messageSize
			Array.Copy(_buffer, _offset, _buffer, 0, lengthToCopy)
			_offset = lengthToCopy
		End Sub
		'this method checks if a complete message is received
		Public Function IsMessageReceived() As Boolean
			If _offset < 4 Then
				Return False
			End If
			Dim sizeToRecieve As Integer = BitConverter.ToInt32(_buffer, 0)
			'check if we have a complete NetworkMessage
			If (_offset - 4) < sizeToRecieve Then
				Return False 'we have not received the complete message yet
			End If
			'we received the complete message and may be more
			Return True
		End Function
	End Class
End Namespace
