Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Net
Imports System.Net.Sockets
Imports TcpMessages

Namespace AdvPolicyServer
	Friend Class TcpClientConnection
		Private _receiveBuffer As PolicyReceiveBuffer
		Private _acceptedSocket As Socket
		Private _receiveSocketArgs As SocketAsyncEventArgs
		Private _policyBuffer() As Byte
		Public Property ReceiveBuffer() As PolicyReceiveBuffer
			Get
				Return _receiveBuffer
			End Get
			Set(ByVal value As PolicyReceiveBuffer)
				_receiveBuffer = value
			End Set
		End Property
		Public Sub New(ByVal acceptedSocket As Socket, ByVal policyBuffer() As Byte)
			Me._acceptedSocket = acceptedSocket
			Me._policyBuffer = policyBuffer
			Me.ReceiveBuffer = New PolicyReceiveBuffer(1024)
			Me._receiveSocketArgs = New SocketAsyncEventArgs()
			Me._receiveSocketArgs.SetBuffer(_receiveBuffer.Buffer, 0, _receiveBuffer.Buffer.Length)
			AddHandler _receiveSocketArgs.Completed, AddressOf ReceiveCallback
		End Sub
		Public ReadOnly Property AcceptedSocket() As Socket
			Get
				Return _acceptedSocket
			End Get
		End Property

		Public Sub ReceiveAsync()
			ReceiveAsync(_receiveSocketArgs)
		End Sub

		Private Sub ReceiveAsync(ByVal socketAsyncEventArgs As SocketAsyncEventArgs)
			If (Not _acceptedSocket.ReceiveAsync(socketAsyncEventArgs)) Then
				ReceiveCallback(_acceptedSocket, socketAsyncEventArgs)
			End If
		End Sub

		Private Sub ReceiveCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.SocketError <> SocketError.Success Then
				Return
			End If
			_receiveBuffer.Offset += e.BytesTransferred
			If _receiveBuffer.IsMessageReceived() Then
				SendPolicy()
			Else
				'adjust the buffer pointer
				e.SetBuffer(_receiveBuffer.Offset, _receiveBuffer.Remaining)
				ReceiveAsync(_receiveSocketArgs)
			End If
		End Sub
		Public Sub SendPolicy()
			SendAsync(_policyBuffer)
		End Sub
		Public Sub SendAsync(ByVal sendBytes() As Byte)
			Dim sendSocketArgs As New SocketAsyncEventArgs()
			sendSocketArgs.SetBuffer(sendBytes, 0, sendBytes.Length)
			AddHandler sendSocketArgs.Completed, AddressOf SendCallback
			SendAsync(sendSocketArgs)
		End Sub

		Private Sub SendAsync(ByVal sendSocketArgs As SocketAsyncEventArgs)
			If (Not _acceptedSocket.SendAsync(sendSocketArgs)) Then
				'call is completed synchronously
				SendCallback(_acceptedSocket, sendSocketArgs)
			End If
		End Sub

		Private Sub SendCallback(ByVal sender As Object, ByVal e As SocketAsyncEventArgs)
			If e.BytesTransferred < e.Count Then
				'some more bytes to send
				e.SetBuffer(e.BytesTransferred, e.Count - e.BytesTransferred)
				SendAsync(e)
			Else
				'we no longer need the connection
				Me.AcceptedSocket.Close()
			End If
		End Sub
	End Class
End Namespace
